/**
 * 向父级元素递归搜索
 * 
 * @param _el       当前所在元素
 * @param tagName   目标标签名称
 * @param className 目标元素样式类
 * @returns 目标元素，找不到为 null
 */
export function up(_el: Element, tagName: string, className: string): Element | null {
    if (tagName && className)
        throw '只能任选一种参数，不能同时传';

    let el: Element = _el.parentNode as Element;
    tagName = tagName && tagName.toUpperCase();

    while (el) {
        if (tagName && el.tagName == tagName)
            return el;

        if (className && el.className && ~el.className.indexOf(className))
            return el;

        el = <Element>el.parentNode;
    }

    return null;
}

/**
 * 日期格式化
 * @author meizz
 * @param date  日期，必须为 Date 类型
 * @param fmt   格式模板
 * @returns 格式化后的字符串
 */
export function dateFormat(date: Date, fmt: string): string {
    let o: { [key: string]: number } = {
        "M+": date.getMonth() + 1,                  // 月份   
        "d+": date.getDate(),                       // 日   
        "h+": date.getHours(),                      // 小时   
        "m+": date.getMinutes(),                    // 分   
        "s+": date.getSeconds(),                    // 秒   
        "q+": Math.floor((date.getMonth() + 3) / 3),// 季度   
        "S": date.getMilliseconds()                 // 毫秒   
    };

    if (/(y+)/.test(fmt))
        fmt = fmt.replace(RegExp.$1, (date.getFullYear() + "").substr(4 - RegExp.$1.length));

    for (var k in o)
        if (new RegExp("(" + k + ")").test(fmt)) {
            let obj = (RegExp.$1.length == 1) ? o[k] : ("00" + o[k]).substr(("" + o[k]).length);
            // @ts-ignore
            fmt = fmt.replace(RegExp.$1, obj);
        }

    return fmt;
}

/**
 * 加载脚本
 * 
 * @param url   脚本地址
 * @param id    脚本元素 id，可选的
 * @param cb    回调函数，可选的
 */
export function loadScript(url: string, id?: string, cb?: (ev: Event) => any): void {
    let script: HTMLScriptElement = document.createElement("script");
    script.src = url;

    if (cb)
        script.onload = cb;

    if (id)
        script.id = id;

    document.getElementsByTagName("head")[0].appendChild(script);
}

/**
 * 并行和串行任务 
 * 
 * @author https://segmentfault.com/a/1190000013265925
 * @param arr 
 * @param finnaly 
 */
export function parallel(arr: [], _finally: Function) {
    let fn: Function, index: number = 0;
    // @ts-ignore
    let statusArr = Array(arr.length).fill().map(() => ({ isActive: false, data: null }));

    let isFinished = function () {
        return statusArr.every((item: any) => item.isActive === true);
    }

    let resolve = function (index: number): Function {
        return function (data: any) {
            statusArr[index].data = data;
            statusArr[index].isActive = true;
            let isFinish = isFinished();

            if (isFinish) {
                let datas = statusArr.map((item: any) => item.data);

                _finally(datas);
            }
        };
    };

    // @ts-ignore
    while ((fn = arr.shift())) {
        fn(resolve(index));// 给 resolve 函数追加参数,可以使用 bind 函数实现,这里使用了柯里化
        index++;
    }
}

/**
 * 函数节流
 * 
 * @author https://www.cnblogs.com/moqiutao/p/6875955.html
 * @param fn 
 * @param delay 
 * @param mustRunDelay 
 */
export function throttle(fn: Function, delay: number, mustRunDelay: number): Function {
    var timer: number, t_start: number;

    return function () {
        var t_curr = +new Date();
        window.clearTimeout(timer);

        if (!t_start)
            t_start = t_curr;

        if (t_curr - t_start >= mustRunDelay) {
            // @ts-ignore
            fn.apply(this, arguments);
            t_start = t_curr;
        } else {
            var args = arguments;
            // @ts-ignore
            timer = window.setTimeout(() => fn.apply(this, args), delay);
        }
    };
}